# %%
import os, time, string, json
import glob
import numpy as np
import pandas as pd
from PIL import Image, ImageFont, ImageDraw
import plotly.express as px
import plotly.graph_objects as go
import plotly.io as pio
pio.templates.default = 'plotly_dark'
from typing import Union, Any, List, Dict, Tuple
from enlighten import Counter

# %%
from image2layout_computer_vision import (
    extract_colors, ColorExtractor, COLOR, ImageTransform
)

# %%
fps = [
    '/Users/felixdo/Documents/Code/AI/image2layout-computer-vision/data/inputs/section_capturenmvqljbvbg.png',
    '/Users/felixdo/Documents/Code/AI/image2layout-computer-vision/data/inputs/section_capturezxspaifrgr.png',
]

fps = glob.glob(os.path.join('/Users/felixdo/Documents/Code/AI/image2layout-computer-vision/data/url', '*.png'))
fps = np.random.choice(fps, 20, replace=False).tolist()
len(fps)

# %%
data = []
color_extractors = []
pbar = Counter(total=len(fps))
for i, fp in enumerate(fps):
    _img = Image.open(fp).convert('RGB')
    n_pixels = int(np.prod(_img.size))
    time_start = time.perf_counter()
    _ce = ColorExtractor(_img)
    color_extractors.append(_ce)
    time_cost = time.perf_counter() - time_start
    data.append({
        'size': _img.size,
        'n_pixels': n_pixels,
        'time_cost': time_cost,
        'color_bg': _ce.color_bg,
        'color_fg': _ce.color_fg,
    })
    del _img
    pbar.update()

color_extractors
df = pd.DataFrame(data)
df

# %%
px.scatter(
    df,
    x='n_pixels',
    y='time_cost',
)

# %%
imgs_anno = [obj.draw_anno() for obj in color_extractors]

# %%
ImageTransform.concatenate(
    [
        _img
        for _color_extractor in color_extractors
        for _img in [
            _color_extractor.image,
            _color_extractor.image_anno,
        ]
    ],
    columns=2
).convert('RGB')

# %%





# %%
import numpy as np
import colorsys

# %%
count = 20
colors_rgb2 = np.random.sample([count, 2, 3])
colors_rgb2 = np.round(colors_rgb2 * 15) / 15
colors_hls2 = np.array([
    [
        colorsys.rgb_to_hls(*v1)
        for v1 in v
    ]
    for v in colors_rgb2
])
colors_l = (colors_rgb2 * [0.2126, 0.7152, 0.0722]).sum(-1)
colors_rgb2.shape, colors_hls2.shape, colors_l.shape

# %%
# b = np.sort(colors_hls2[:, :, 1] + 0.05, axis=-1)
b = np.sort(colors_l + 0.05, axis=-1)
contrast_l = b[:, 1] / b[:, 0]

# %%
s = 40
s_text = 80
# img_comp = Image.new(
#     'RGB',
#     (s * 3, s * count),
# )
img_comp_np = np.zeros([s * count, s * 2 + s_text, 3], np.uint8)
for i in range(count):
    for j in range(2):
        img_comp_np[
            s * i : s * (i + 1),
            s * j : s * (j + 1),
        ] = colors_rgb2[i, j] * 255

img_comp = Image.fromarray(img_comp_np)
draw = ImageDraw.Draw(img_comp)
font = ImageFont.truetype('/Users/felixdo/Documents/Code/AI/image2layout-computer-vision/templates/OpenSans_Condensed-Medium.ttf', 28)
for i in range(count):
    draw.text(
        (s * 2 + 2, s * i),
        text=f'{contrast_l[i]:.2f}',
        fill=(255, 255, 255),
        # spacing=20,
        font=font,
    )

img_comp

# %%
def convert_rgb2lum(colors_rgb):
    shape = colors_rgb.shape
    _colors_rgb = colors_rgb.reshape(-1)
    # print('shape', _colors_rgb.shape)
    _colors_rgb_inter = ((_colors_rgb+0.055)/1.055) ** 2.4
    _colors_rgb_inter[_colors_rgb <= 0.03928] = (_colors_rgb / 12.92)[_colors_rgb <= 0.03928]
    colors_lum = (_colors_rgb_inter.reshape(shape) * [0.2126, 0.7152, 0.0722]).sum(-1)
    return colors_lum

# %%
# count = 200
# colors_rgb = np.random.sample([count, 3])
# colors_rgb

colors_rgb = np.array([
    [r, g, b]
    for r in np.linspace(0, 1, 16)
    for g in np.linspace(0, 1, 16)
    for b in np.linspace(0, 1, 16)
])
count = len(colors_rgb)

lum_manual = convert_rgb2lum(colors_rgb)
lum_colorsys_l = np.array([colorsys.rgb_to_hls(*v) for v in colors_rgb])[:, 1]
lum_colorsys_v = np.array([colorsys.rgb_to_hsv(*v) for v in colors_rgb])[:, 2]
colors_yiq = COLOR.rgb2yiq(colors_rgb * 255)
lum_y = colors_yiq[:, 0]
count

# %%
colors_data = []
for i in range(count):
    colors_data.append({
        'color': COLOR.rgb2hex(colors_rgb[i] * 255),
        'lum_colorsys_l': lum_colorsys_l[i],
        'lum_colorsys_v': lum_colorsys_v[i],
        'lum_manual': lum_manual[i],
        'lum_y': lum_y[i],
        'color_yiq': colors_yiq[i],
    })

colors_df = pd.DataFrame(colors_data)

def get_random_jitter(n=1, amount=0.1):
    return (np.random.sample(n) * 2 - 1) * amount

jitter_amount = 0.035
# colors_df['lum_colorsys_l'] = np.clip(colors_df['lum_colorsys_l'] + get_random_jitter(count, jitter_amount), 0, 1)
# colors_df['lum_colorsys_v'] = np.clip(colors_df['lum_colorsys_v'] + get_random_jitter(count, jitter_amount), 0, 1)
# colors_df['lum_manual'] = np.clip(colors_df['lum_manual'] + get_random_jitter(count, jitter_amount), 0, 1)
colors_df

# %%
s = 1000
# fig = px.scatter(
#     colors_df,
#     # x='lum_colorsys_v',
#     # y='lum_colorsys_l',
#     # x='lum_colorsys_l',
    
#     x='lum_y',
#     y='lum_colorsys_v',
#     # y='lum_manual',
    
#     color='color',
#     # color=colors_df['color'].to_list(),
#     color_discrete_map={
#         v: v
#         for v in colors_df['color'].to_list()
#     },
#     range_x=[0, 1],
#     range_y=[0, 1],
# )

_colors_columns = [
    'lum_colorsys_v',
    'lum_colorsys_l',
    'lum_manual',
    'lum_y',
]
_colors_list = [
    colors_df[col]
    for col in _colors_columns
]
_colors_list_count = len(_colors_list)
x_spacing = 1 / _colors_list_count
x_spacing, (np.arange(_colors_list_count) + 0.5) / _colors_list_count

# %%
fig = px.scatter(
    colors_df,
    # x='lum_colorsys_v',
    # y='lum_colorsys_l',
    # x='lum_colorsys_l',
    
    x=np.tile(
        ((np.arange(_colors_list_count) + 0.5) / _colors_list_count).reshape(-1, 1), [1, count]
    ).reshape(-1) + get_random_jitter(count * _colors_list_count, x_spacing / 2 * 0.85),
    y=np.concatenate(_colors_list),
    
    color=colors_df['color'].to_list() * _colors_list_count,
    # color=colors_df['color'].to_list(),
    color_discrete_map={
        v: v
        for v in colors_df['color'].to_list()
    },
    range_x=[0, 1],
    range_y=[0, 1],
)
fig.update_layout(
    showlegend=False,
    width=s, height=s,
    title='Luminance' + ' | '.join(_colors_columns),
).update_yaxes(
    scaleanchor="x",
    scaleratio=1,
)

# %%
px.scatter_3d(
    x=colors_yiq[:, 0],
    y=colors_yiq[:, 1],
    z=colors_yiq[:, 2],
    color=colors_df['color'].to_list(),
    # color=colors_df['color'].to_list(),
    color_discrete_map={
        v: v
        for v in colors_df['color'].to_list()
    },
)